searchData={"items":[{"type":"module","doc":"TypedEctoSchema provides a DSL on top of `Ecto.Schema` to define schemas with typespecs without\nall the boilerplate code.","title":"TypedEctoSchema","ref":"TypedEctoSchema.html"},{"type":"module","doc":"Normally, when defining an `Ecto.Schema` you probably want to define:\n  * the schema itself\n  * the list of enforced keys (which helps reducing problems)\n  * its associated type (`Ecto.Schema` doesn't define it for you)\n\nIt ends up in something like this:\n    defmodule Person do\n      use Ecto.Schema\n\n      @enforce_keys [:name]\n\n      schema \"people\" do\n        field(:name, :string)\n        field(:age, :integer)\n        field(:happy, :boolean, default: true)\n        field(:phone, :string)\n        belongs_to(:company, Company)\n        timestamps(type: :naive_datetime_usec)\n      end\n\n      @type t() :: %__MODULE__{\n              __meta__: Ecto.Schema.Metadata.t(),\n              id: integer() | nil,\n              name: String.t(),\n              age: non_neg_integer() | nil,\n              happy: boolean(),\n              phone: String.t() | nil,\n              company_id: integer() | nil,\n              company: Company.t() | Ecto.Association.NotLoaded.t() | nil,\n              inserted_at: NaiveDateTime.t(),\n              updated_at: NaiveDateTime.t()\n            }\n    end\n\n  This is problematic for a a lot of reasons, summing up:\n\n  - A lot of repetition. Field names appear in 3 different places, so in order to understand one\n    field, a reader needs to go up and down the code to get that.\n  - Ecto has some \"hidden\" fields that are added behind the scenes to the struct, such as the\n    primary key `id`, the foreign key `company_id`, the timestamps and the `__meta__` field for\n    schemas. Knowing all those rules can be hard to remember and would probably be easily\n    forgotten when changing the schema. Also, Ecto has strange types for associations and metadata that\n    need to be remembered.\n\nAll of this makes this process extremely repetitive and error prone. Sometimes, you want to\nenforce factory functions to control defaults in a better way, you would probably add all fields\nto `@enforce_keys`. This would make the `@enforce_keys` big and repetitive, once again.\n\nThis module aims to help with that, by providing some syntax sugar that allow you to define this\nin a more compact way.\n\n    defmodule Person do\n      use TypedEctoSchema\n\n      typed_schema \"people\" do\n        field(:name, :string, enforce: true, null: false)\n        field(:age, :integer) :: non_neg_integer() | nil\n        field(:happy, :boolean, default: true, null: false)\n        field(:phone, :string)\n        belongs_to(:company, Company)\n        timestamps(type: :naive_datetime_usec)\n      end\n    end\n\nThis is way simpler and less error prone. There is a lot going under the hoods here.","title":"Rationale - TypedEctoSchema","ref":"TypedEctoSchema.html#module-rationale"},{"type":"module","doc":"All ecto macros are called under the hood with the options you pass, with exception of a few\nadded options:\n\n- `:null` - when `true`, adds a `| nil` to the typespec. Default is `true`. Has no effect on\n  `has_one/3` because it can always be `nil`. On `belongs_to/3` only add `| nil` to the\n  underlying foreign key.\n- `:enforce` - when `true` adds the field to the `@enforce_keys`. Default is `false`","title":"Extra Options - TypedEctoSchema","ref":"TypedEctoSchema.html#module-extra-options"},{"type":"module","doc":"When calling `typed_schema/3` or `typed_embedded_schema/2` you can pass some options, as\ndefined:\n\n- `:null` - Set the default `:null` field option, which normally is true. Note that it is still\n  can be overwritten by passing `:null` to the field itself.\n  Also, `embeds_many` and `has_many` can never be null, because they are always initialized to\n  empty string, so they never receive the `| nil` on the typespec.\n  In addition to that, `has_one/3` and `belongs_to/3` always receive `| nil` because the related\n  schema may be deleted from the repo so it is safe to always assume they can be `nil`.\n- `:enforce` - When `true`, enforces all fields unless they explicitly set `enforce: false` or\n  defines a default (`default: value`), since it makes no sense to have a default value for an\n  enforced field.\n- `:opaque` - When `true` makes the generated type `t` be an opaque type.","title":"Schema Options - TypedEctoSchema","ref":"TypedEctoSchema.html#module-schema-options"},{"type":"module","doc":"TypedEctoSchema does its best job to guess the typespec for the field. It does so by following\nthe Elixir types as defined in [`Ecto.Schema`](https://hexdocs.pm/ecto/Ecto.Schema.html#module-primitive-types).\nFor custom `Ecto.Type` and related schemas (embedded and associations), which are always a\nmodule, it assumes the schemas has a type `t/0` defined, so for a schema called `MySchema`, it\nwill assume the type is `MySchema.t/0`, which is also, the default type generated by this\nlibrary.","title":"Type Inference - TypedEctoSchema","ref":"TypedEctoSchema.html#module-type-inference"},{"type":"module","doc":"If for somereason you want to narrow the type or the automatic type inference is incorrect,\nthe `::` operator allows the typespec to be overriden.\nThis is done as you would when defining typespecs.\n\nSo, for example, instead of\n\n```elixir\nfield(:my_int, :integer)\n```\n\nWhich would generate a `integer() | nil` typespec, you can:\n\n```elixir\nfield(:my_int, :integer) :: non_neg_integer() | nil\n```\n\nAnd then have a `non_neg_integer()` type for it.","title":"Overriding the Typespec for a field - TypedEctoSchema","ref":"TypedEctoSchema.html#module-overriding-the-typespec-for-a-field"},{"type":"module","doc":"Ecto generates some fields for you in a lot of cases, they are:\n\n- For primary keys\n- When using a `belongs_to/3`\n- When calling `timestamps/1`\n\nThe `__meta__` typespec is automatically generated and cannot be overriden. That is because\nthere is no point on overriding it.","title":"Non explicit generated fields - TypedEctoSchema","ref":"TypedEctoSchema.html#module-non-explicit-generated-fields"},{"type":"module","doc":"Primary keys are generated by default and can be customized by the `@primary_key` module\nattribute, just as defined by Ecto. We handle `@primary_key` the same way we handle `field/3`, so you\ncan pass the same field options to it.\n\nHowever, if you want to customize the type, you need to set `@primary_key false` and define a\nfield with `primary_key: true`.","title":"Primary Keys - TypedEctoSchema","ref":"TypedEctoSchema.html#module-primary-keys"},{"type":"module","doc":"`belongs_to` generates an underlying foreign key that is dependent on a few Ecto options, as\ndefined on [`Ecto.Schema`](https://hexdocs.pm/ecto/Ecto.Schema.html#belongs_to/3-options).\n\nThe options we are interested in are `:foreign_key`, `:define_field` and `:type`\n\nWhen `:null` is passed,  it will add `| nil` to the generated `foreign_key`'s typespec.\n\nThe `:enforce` option enforces the association field instead.\nIf you want to `:enforce` the foreign key to be set, you should probably pass `define_field:\nfalse` and define the foreign key by hand, setting another `field/3`, the same way as\ndescribed by Ecto's doc.","title":"Belongs To - TypedEctoSchema","ref":"TypedEctoSchema.html#module-belongs-to"},{"type":"module","doc":"In the case of the timestamps, we currently don't allow overriding the type by using the `::` operator.\nThat being said, however, we define the type of the fields using the `:type` option\n([as defined by Ecto doc](https://hexdocs.pm/ecto/Ecto.Schema.html#timestamps/1-options))","title":"Timestamps - TypedEctoSchema","ref":"TypedEctoSchema.html#module-timestamps"},{"type":"macro","doc":"Replaces `Ecto.Schema.embedded_schema/1`","title":"TypedEctoSchema.typed_embedded_schema/2","ref":"TypedEctoSchema.html#typed_embedded_schema/2"},{"type":"macro","doc":"Replaces `Ecto.Schema.schema/2`","title":"TypedEctoSchema.typed_schema/3","ref":"TypedEctoSchema.html#typed_schema/3"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}